!*****************************************************************
!                        DEM_spots.f90  ver. 2.1  12/6/2023
!         回折斑点像の対を見つけて二重露光法で回折角を求めるプログラム
!              K. Suzuki, Niigata University
!*****************************************************************
! 

program DEM_spots

  implicit none
  integer::n,dir
  real*8::w,h  !beam size, mm
  real*8::t !試験片板厚, mm
  real*8:: ey=4.33D-04  !  降伏ひずみ


  print *, "hoop またはradialのフォルダーにDEM_line.f90をおいてください．"
  print *, "この下にP1, P2フォルダーがあり，そこにspot_i.dat群をおいてください．"
  call SYSTEM('ls ./P2')
  print *, "処理停止のspot_i.datの番号(loop)"
  read(5,*) n
  
  print *,"周方向(hoop): 1,  半径方向(radial):2"
  print *, "測定方向の番号を入力してください．"
  read(5,*) dir
  
  print *,"ビームサイズ(mm)  w, h を入力してください．"
  read(5,*) w,h
  print *,"試験片板厚t (mm) を入力してください．"
  read(5,*) t
  
  
  if (dir == 1) then
    call hoop(n,w,h,t)
  end if
  
  if (dir == 2) then
    call radial(n,w,h,t)
  end if

  ! 周方向hoopのひずみを計算
  if(dir == 1) then
    call h_strain(n,ey)
  end if
  
  
  ! 半径方向radialひずみを計算
  if(dir == 2) then
    call r_strain(n,ey)
  end if
  
  
end program DEM_spots

!======================================
!周方向の回折計算
subroutine hoop(n,w,h,t)
  implicit none
  
  character:: P1_file*30,P2_file*30,dir1*30,dir2*30,No*5
  integer:: i,j,k,loop,m1,m2,n, n1,n2,cnt
  real*8:: x1(1:1000),y1,z1(1:1000)
  real*8:: x2(1:1000),y2,z2(1:1000)
  real*8:: l(1:1000),r1,r2,th,phi,pi=3.1415926
  real*8:: xa,za,tth,xc,yc,zc,y21,ct
  real*8:: w,h,t
  
  open(unit=12,file='temp_h.txt',status='unknown')     !*************周方向の回折データ
  open(unit=14,file='hoop_n.txt',status='unknown')
    
  do loop = 0,n
  !                                 斑点データの読み込み
    write(No,'(I5)') loop
    P1_file="./P1/spot_" // trim(adjustl(No)) //".dat"
    P2_file="./P2/spot_" // trim(adjustl(No)) //".dat"
    cnt=0
    
    open(unit=11,file=P1_file,status='old')
      read(11,*) m1
      do i=1,m1
        read(11,*) j,x1(j),y1,z1(j)
      end do
    close(11)
    
    open(unit=11,file=P2_file,status='old')
      read(11,*) m2
      do i=1,m2
        read(11,*) j,x2(j),y2,z2(j)
      end do
    close(11)
    print *,P1_file,P2_file
  !                               交点の計算
    do i=1,m2
      ! P2の斑点抽出
      if((abs(x2(i))<38.11).and.(z2(i)>533.05.and.z2(i)<546.41)) then
        xa=x2(i)*y1/y2; za=z2(i)*y1/y2

        do j=1,m1
        !    対応するP1の斑点を抽出
         if((abs(xa-x1(j))<1.0).and.(abs(za-z1(j))<1.0)) then
          r1=sqrt(x1(j)**2+z1(j)**2)
          r2=sqrt(x2(i)**2+z2(i)**2)
          tth=atan((r2-r1)/(y2-y1))
          
          ct=1./tan(tth)
          y21=y2-y1
          xc=x1(j)-ct**2/y21**2*((x2(i)-x1(j))*x1(j) &
                              +(z2(i)-z1(j))*z1(j))*(x2(i)-x1(j))
          yc=y1-ct**2/y21*((x2(i)-x1(j))*x1(j)+(z2(i)-z1(j))*z1(j))

          zc=z1(j)-ct**2/y21**2*((x2(i)-x1(j))*x1(j) &
                              +(z2(i)-z1(j))*z1(j))*(z2(i)-z1(j))
          
          if(abs(xc)<w/2.0 .and. abs(yc)<t/2.0 .and. abs(zc)<h/2.0) then
            write(6,101) loop,xc,yc,zc,tth*180/pi
            write(12,101) loop,xc,yc,zc,tth*180/pi
            cnt=cnt+1
          end if
         end if
        end do
        
      end if
    
    end do
    
    write(14,*) loop,cnt ;write(6,*) loop,cnt
    
  end do  !loop=0,n

  close(12)
  close(14)
  
 101 format(I4,2X,F6.3,2X,F6.3,2X,F6.3,2X,F7.4)
end subroutine hoop

!======================================
!         半径方向の回折計算
subroutine radial(n,w,h,t)
  implicit none
  
  character:: P1_file*30,P2_file*30,dir1*30,dir2*30,No*5
  integer:: i,j,k,loop,m1,m2,n, n1,n2,cnt
  real*8:: x1(1:1000),y1,z1(1:1000)
  real*8:: x2(1:1000),y2,z2(1:1000)
  real*8:: l(1:1000),r1,r2,th,phi,pi=3.1415926
  real*8:: xa,za,tth,xc,yc,zc,y21,ct
  real*8:: w,h,t
    
  open(unit=12,file='temp_r.txt',status='unknown')     !*************半径方向の回折データ
  open(unit=14,file='radial_n.txt',status='unknown')
  
  do loop = 0,n
  !                                 斑点データの読み込み
    write(No,'(I5)') loop
    P1_file="./P1/spot_" // trim(adjustl(No)) //".dat"
    P2_file="./P2/spot_" // trim(adjustl(No)) //".dat"
    cnt=0
    
    open(unit=11,file=P1_file,status='old')
      read(11,*) m1
      do i=1,m1
        read(11,*) j,x1(j),y1,z1(j)
      end do
    close(11)
    
    open(unit=11,file=P2_file,status='old')
      read(11,*) m2
      do i=1,m2
        read(11,*) j,x2(j),y2,z2(j)
      end do
    close(11)
    print *,P1_file,P2_file
  !                               交点の計算
    do i=1,m2
      ! P2の斑点抽出
      if((abs(z2(i))<38.11).and.(x2(i)>533.05.and.x2(i)<546.41)) then
        xa=x2(i)*y1/y2; za=z2(i)*y1/y2

        do j=1,m1
        !    対応するP1の斑点を抽出
         if((abs(xa-x1(j))<1.0).and.(abs(za-z1(j))<1.0)) then
          r1=sqrt(x1(j)**2+z1(j)**2)
          r2=sqrt(x2(i)**2+z2(i)**2)
          tth=atan((r2-r1)/(y2-y1))
          
          ct=1./tan(tth)
          y21=y2-y1
          xc=x1(j)-ct**2/y21**2*((x2(i)-x1(j))*x1(j) &
                              +(z2(i)-z1(j))*z1(j))*(x2(i)-x1(j))
          yc=y1-ct**2/y21*((x2(i)-x1(j))*x1(j)+(z2(i)-z1(j))*z1(j))

          zc=z1(j)-ct**2/y21**2*((x2(i)-x1(j))*x1(j) &
                              +(z2(i)-z1(j))*z1(j))*(z2(i)-z1(j))
          !90度回転しているので，制限するスリットの軸が入れ替わる．
          if(abs(xc)<h/2.0 .and. abs(yc)<t/2.0 .and. abs(zc)<w/2.0) then
            write(6,101) loop,xc,yc,zc,tth*180/pi
            write(12,101) loop,xc,yc,zc,tth*180/pi
            cnt=cnt+1
          end if
         end if
        end do
        
      end if
    
    end do
    
    write(14,*) loop,cnt ;write(6,*) loop,cnt

  end do  !loop=0,n
  
  close(12)
  close(14)

 101 format(I4,2X,F6.3,2X,F6.3,2X,F6.3,2X,F7.4)
end subroutine radial

!  *********  周方向hoopのひずみを計算 ******************
subroutine h_strain(loop,ey)
!  ***************************************************

  implicit none
  
  !character:: filename1*30,filename2*30
  integer:: i,j,k, m, n,cnt,loop,c_p,c_m
  integer:: n_spot(0:2000)
  real*8:: xc(1:400),yc(1:400),zc(1:400),tth(1:400)
  real*8:: tth0,th0,s,d1,d2,d3,d2th, pi=3.1415926
  real*8:: s_p,s_m,tth_p,tth_m,th_p,th_m
  real*8:: ey !  降伏ひずみ
  real*8::th2_max,th2_min
  !-------------------周方向の全回折角の平均(無ひずみの回折角)を求める
  
  open(unit=11,file='temp_h.txt',status='old')
    cnt=0; s=0.0

    do       !　読み込みにはこんな命令の書き方も可能．
      read(11,*,end=999) i,d1,d2,d3,d2th
      s=s+d2th; cnt=cnt+1
    end do
    
 999 close(11)
  
  tth0 = s/real(cnt)    !************ 周方向の無ひずみの回折角************

  print *,"Strain free 2θ=",tth0
  
  open(unit=12,file="hoop.txt",status="unknown")
    write(12,*) tth0
  open(unit=11,file="temp_h.txt",status="old")
    do
      read(11,*,end=998) i,d1,d2,d3,d2th
      write(12,102)  i,d1,d2,d3,d2th
    end do
  
  
 998 close(11);close(12)
  
    th0=tth0/2.0*pi/180.0   !rad
  
  !                -------------- 周方向の各位置のひずみを計算する --------
  
    open(unit=11,file='hoop_n.txt',status='old')
    open(unit=13,file='hoop.txt',status='old')
    open(unit=15,file='strain_h.dat',status='unknown')
    
    read(13,*) tth0
    write(15,*) 'strain free θ_0=', th0*180/pi
    write(15,*) '#No.           strain(-y),            strain(+y) '
     
  do i=0,loop
    read(11,*) j,m
    
    !  周方向の各位置の測定結果の読み込み
    do k=1,m,1
      read(13,*) j,xc(k),yc(k),zc(k),tth(k)
    end do

    ! 周方向の-y, +yの回折角度の計算(平均)，ひずみの計算
    
    c_p=0; c_m=0   !+counter, -counter  初期化
    tth_p=0.0; tth_m=0.0 
    
    th2_max=asin(sin(th0)/(1.0 - ey))*360./pi  !deg
    th2_min=asin(sin(th0)/(1.0 + ey))*360./pi   !deg
    
    do k=1,m,1
      if(th2_min < tth(k) .and. tth(k) < th2_max) then
        if(yc(k) < 0) then
          tth_m = tth_m + tth(k) ; c_m=c_m+1
        else
          tth_p = tth_p + tth(k) ; c_p=c_p+1
        end if
     end if
    end do
    
    
    
    if(c_p == 0) then
        th_p= th0  !rad
      else
        tth_p=tth_p/real(c_p)
        th_p=tth_p/2.0*pi/180 !rad
    end if
      
    if(c_m == 0) then
        th_m= th0  !rad
      else
        tth_m=tth_m/real(c_m)
        th_m=tth_m/2.0*pi/180 !rad
    end if
    
    if(tth_p == 0.0) then
      th_p = th0
    end if
    
    if(tth_m ==0.0) then
      th_m= th0
    end if
    
    !           周方向のひずみs_p, s_m の計算と出力
    s_p =sin(th_p)/sin(th0) - 1.0
    s_m =sin(th_m)/sin(th0) - 1.0
    
    write(15,101) i,s_m,s_p
      write(6,101) i,s_m,s_p
  end do !i=0,loop
  
  close(11);close(13);close(15)

 102 format(I4,2X,F6.3,2X,F6.3,2X,F6.3,2X,F7.4)
 101 format(I5,2(2x,D15.5))
end subroutine  h_strain


!  *********  半径方向hoopのひずみを計算 ******************
subroutine r_strain(loop,ey)
!  ***************************************************

  implicit none
  
  !character:: filename1*30,filename2*30
  integer:: i,j,k, m, n,cnt,loop,c_p,c_m
  integer:: n_spot(0:2000)
  real*8:: xc(1:400),yc(1:400),zc(1:400),tth(1:400)
  real*8:: tth0,th0,s,d1,d2,d3,d2th, pi=3.1415926
  real*8:: s_p,s_m,tth_p,tth_m,th_p,th_m
  real*8:: ey !降伏ひずみ
  real*8::th2_max,th2_min
  !-------------------半径方向の全回折角の平均(無ひずみの回折角)を求める
  
  open(unit=11,file='temp_r.txt',status='old')
    cnt=0; s=0.0
    do       !　読み込みにはこんな命令の書き方も可能．
      read(11,*,end=999) i,d1,d2,d3,d2th
      s=s+d2th; cnt=cnt+1
    end do
    
 999 close(11)
  
  tth0 = s/real(cnt)    !************ 半径方向の無ひずみの回折角************
  
    print *,"Strain free 2θ=",tth0
  
  open(unit=12,file="radial.txt",status="unknown")
    write(12,*) tth0
  open(unit=11,file="temp_r.txt",status="old")
    do
      read(11,*,end=998) i,d1,d2,d3,d2th
      write(12,102)  i,d1,d2,d3,d2th
    end do
  
  
 998 close(11);close(12)
  
    th0=tth0/2.0*pi/180.0   !rad
  
  !                -------------- 半径方向　各位置のひずみを計算する --------
  
    open(unit=11,file='radial_n.txt',status='old')
    open(unit=13,file='radial.txt',status='old')
    open(unit=12,file='strain_r.dat',status='unknown')
    
        read(13,*) tth0
    write(12,*) 'strain free θ_0=', th0*180/pi
    write(12,*) '#No.           strain(-y),            strain(+y) '
     
  do i=0,loop
    read(11,*) j,m
    
    !  半径方向の各位置の測定結果の読み込み
    do k=1,m,1
      read(13,*) j,xc(k),yc(k),zc(k),tth(k)
    end do

    ! 半径方向の-y, +yの回折角度の計算(平均)，ひずみの計算
    
    c_p=0; c_m=0   !+counter, -counter  初期化
    tth_p=0.0; tth_m=0.0 
    th2_max=asin(sin(th0)/(1.0 - ey))*360./pi  !deg
    th2_min=asin(sin(th0)/(1.0 + ey))*360./pi   !deg
    
    do k=1,m,1
      if(th2_min < tth(k) .and. tth(k) < th2_max) then
        if(yc(k) < 0) then
          tth_m = tth_m + tth(k) ; c_m=c_m+1
        else
          tth_p = tth_p + tth(k) ; c_p=c_p+1
        end if
      end if
    end do
    
    if(c_p == 0) then
        th_p= th0  !rad
      else
        tth_p=tth_p/real(c_p)
        th_p=tth_p/2.0*pi/180 !rad
    end if
      
    if(c_m == 0) then
        th_m= th0  !rad
      else
        tth_m=tth_m/real(c_m)
        th_m=tth_m/2.0*pi/180 !rad
    end if
    
    if(tth_p == 0.0) then
      th_p = th0
    end if
    
    if(tth_m ==0.0) then
      th_m= th0
    end if
    
    !           半径方向のひずみs_p, s_m の計算と出力
    s_p =sin(th_p)/sin(th0) - 1.0
    s_m =sin(th_m)/sin(th0) - 1.0
    
    write(12,101) i,s_m,s_p
      write(6,101) i,s_m,s_p
  end do !i=0,loop
  
  close(11);close(13);close(12)

 102 format(I4,2X,F6.3,2X,F6.3,2X,F6.3,2X,F7.4)
 101 format(I5,2(2x,D15.5))
end subroutine  r_strain