!****************************************************************
!      experimental.f90 
!  検出器座標(x,y)から実験室座標系(x1,y1,z1), (x2,y2,z2)への変換
!
!      K. Suzuki,  Niigata University
!****************************************************************


program experimantal

  implicit none
  
  character:: readfile*30,writefile*30,dir1*30,dir2*30
  integer i, j, k, m, n, h,loop
  real*8:: x1(1:1000),y1(1:1000),z1(1:1000) ! P1の実空間座標
  real*8:: x2(1:1000),y2(1:1000),z2(1:1000) ! P2の実空間座標
  real*8:: x(1:1000),y(1:1000) !検出器座標
  real*8:: l0,l,tth,a, pi=3.1415926, x01,y01,x02,y02
  
  ! ---------- experimental data ----------------
  l0=410.29; l=400.0 !カメラ長L0,  検出器間距離L
  tth=38.5 *pi/180.0 !回折計2θアーム角度
  a=0.172  !PILATUS ピクセルサイズ
  x01 = 263.592; y01 = 536.597   ! P1ビームセンター
  x02 = 260.446; y02 = 538.411   ! P2ビームセンター
  
  print *,"TextImageフォルダーにいますか．"
  print *,'P1h = 1, P2h = 2, P1r = 3, P2r = 4'
  print *,'読み込んだデータのタイプの番号を入れてください．'
  read(5,*) m
  
  print *,"フォルダ 0 から処理します．停止するフォルダーの番号を入力してください．"
  read(5,*) k !停止するフォルダーの番号
  
  
  if (m == 1) then !------------------ P1hの場合
  
    call SYSTEM("mkdir P1")   ! P1フォルダ作成
  
    do loop=0,k   ! 測定点数kのループ
    
      write(dir1,'(I5)') loop
      readfile="./" // trim(adjustl(dir1)) // "/spot_list.txt"
      
      open(unit=11,file=readfile,status='old')
      read(11,*) n ! 各位置の斑点数 n
      do i=1,n
        read(11,*) h, x(i),y(i)  !検出器座標
      end do
      close(11)
      
      do i=1,n  ! 実空間座標に変換
        x1(i)= a*(x(i)-x01)
        y1(i)= l0*cos(tth)
        z1(i)= l0*sin(tth) - a*(y(i)-y01)/cos(tth)
      end do
      
      writefile= "./P1/spot_" // trim(adjustl(dir1)) // ".dat"
      
      open(unit=12,file=writefile, status='unknown')
      write(12,*) n
      do i=1,n
        write(12,102) i,x1(i),y1(i),z1(i)
      end do
      close(12)
      
    end do   ! 測定点数kのループ

  end if  !　P1hの場合
  
  if (m == 2) then !------------------ P2hの場合
  
    call SYSTEM("mkdir P2")   ! P2フォルダ作成
  
    do loop=0,k   ! 測定点数kのループ
    
      write(dir1,'(I5)') loop
      readfile="./" // trim(adjustl(dir1)) // "/spot_list.txt"
      
      open(unit=11,file=readfile,status='old')
      read(11,*) n ! 各位置の斑点数 n
      do i=1,n
        read(11,*) h, x(i),y(i)  !検出器座標
      end do
      close(11)
      
      do i=1,n   ! 実空間座標に変換
        x2(i)= a*(x(i)-x02)
        y2(i)= (l0 + l)*cos(tth)
        z2(i)= (l0 + l)*sin(tth) - a*(y(i)-y02)/cos(tth)
      end do
      
      writefile= "./P2/spot_" // trim(adjustl(dir1)) // ".dat"
      
      open(unit=12,file=writefile, status='unknown')
      write(12,*) n
      do i=1,n
        write(12,102) i,x2(i),y2(i),z2(i)
      end do
      close(12)
      
    end do   ! 測定点数kのループ
  end if  !　P2hの場合
  
  if (m == 3) then !------------------ P1rの場合
  
    call SYSTEM("mkdir P1")   ! P1フォルダ作成
  
    do loop=0,k   ! 測定点数kのループ
    
      write(dir1,'(I5)') loop
      readfile="./" // trim(adjustl(dir1)) // "/spot_list.txt"
      
      open(unit=11,file=readfile,status='old')
      read(11,*) n ! 各位置の斑点数 n
      do i=1,n
        read(11,*) h, x(i),y(i)  !検出器座標
      end do
      close(11)
      
      do i=1,n  ! 実空間座標に変換
        x1(i)= l0*sin(tth) - a*(y(i)-y01)/cos(tth)
        y1(i)= l0*cos(tth)
        z1(i)= -a*(x(i)-x01)
      end do
      
      writefile= "./P1/spot_" // trim(adjustl(dir1)) // ".dat"
      
      open(unit=12,file=writefile, status='unknown')
      write(12,*) n
      do i=1,n
        write(12,102) i,x1(i),y1(i),z1(i)
      end do
      close(12)
      
    end do   ! 測定点数kのループ

  end if  !　P1rの場合
  
  
  if (m == 4) then !------------------ P2rの場合
  
    call SYSTEM("mkdir P2")   ! P2フォルダ作成
  
    do loop=0,k   ! 測定点数kのループ
    
      write(dir1,'(I5)') loop
      readfile="./" // trim(adjustl(dir1)) // "/spot_list.txt"
      
      open(unit=11,file=readfile,status='old')
      read(11,*) n ! 各位置の斑点数 n
      do i=1,n
        read(11,*) h, x(i),y(i)  !検出器座標
      end do
      close(11)
      
      do i=1,n   ! 実空間座標に変換
        x2(i)= (l0 + l)*sin(tth) - a*(y(i)-y02)/cos(tth)
        y2(i)= (l0 + l)*cos(tth)
        z2(i)= -a*(x(i)-x02)
      end do
      
      writefile= "./P2/spot_" // trim(adjustl(dir1)) // ".dat"
      
      open(unit=12,file=writefile, status='unknown')
      write(12,*) n
      do i=1,n
        write(12,102) i,x2(i),y2(i),z2(i)
      end do
      close(12)
      
    end do   ! 測定点数kのループ
  end if  !　P2rの場合

 102 format(I5,F12.5,F12.5,F12.5)
end program experimantal